'use strict';

var normalizeValidator = require('./normalize-validator');

/**
 * @param {String} functionName
 * @param {Function} functionTemplate
 * @param {NodePath} functionPath
 * @param {Object} jsDoc
 * @param t
 */
module.exports = function insertParametersAssertion(functionName, functionTemplate, functionPath, jsDoc, t) {
    var parameters = jsDoc.parameters;
    var functionParameters = functionPath.get('params');
    var functionBody = functionPath.get('body');

    if (!functionBody.isBlockStatement()) {
        return;
    }

    iterateThroughArrayOfNodes(functionParameters);

    /**
     * @param {Array<NodePath>} nodes
     */
    function iterateThroughArrayOfNodes(nodes) {
        var path = void 0,
            node = void 0,
            name = void 0;
        var type = void 0;

        for (var index = nodes.length - 1; index >= 0; index--) {
            path = nodes[index];
            node = path.node;

            if (path.isAssignmentPattern()) {
                node = node.left;
            } else if (path.isRestElement()) {
                node = node.argument;
            } else if (path.isObjectProperty()) {
                node = node.key;
            } else if (path.isObjectPattern()) {
                iterateThroughArrayOfNodes(path.get('properties'));
            }

            name = node.name;
            type = parameters[name];

            if (!type) {
                continue;
            }

            functionBody.unshiftContainer('body', functionTemplate(functionName, name, node, normalizeValidator(functionPath, type, t)));
        }
    }
};